package edu.cmu.cs.able.probes.RemosProbe;

import java.io.*;
import java.util.*;
import java.net.*;
import java.text.*;
import java.lang.*;

import remulac.remos.*;
import remulac.ModelerJ.*;

/**
 * Probe is responsible for the probing work by making use of Remos.
 * Probe is deployed by ProbeManager.
 */
public class Probe extends Thread {
	/** ID of the probe */
	ProbeID id;
	/** probe manager that creates this probe */
	ProbeManager pm;

	/** Environment used by collector */
        String commEnv;
	/** Remos modeler, through which to do monitoring work */
        Modeler myModeler;
	/** Remos style ID of the end hosts that the probe is monitoring */
	NodeId srcId, dstId;

	/** probing frequency */
	double frequence;
	/** interval between two consecutive probing */
	long interval;
	/** Probe working status, initial state is non-active, that is, do not probing */
	boolean active = false;

	boolean debugf = true;

	/**
	 * Create a probe, initialize it and get Remos modeler interface.
	 * @param pm point to the probe manager that create this probe
	 */
	Probe(ProbeManager pm) {
	    debugln("Constructor: start");

	    this.pm = pm;

            InetAddress addr;

            try {
                addr = InetAddress.getLocalHost();
	    	id = new ProbeID(addr);
		debugln("get id:"+id.toString());
            } catch (UnknownHostException ex) {
                 System.out.println("InetAddress getLocalHost exception");
            }

	    try {
            	commEnv = System.getProperty("COLLECTOR");
            	// myModeler = Modeler.start(id.toString());
            	myModeler = new Modeler(id.toString());
	    } catch (RemosException ex) {
		System.err.println(ex);
	    } 

	    // this.start();

	    debugln("Constructor: done.");
	};

	/**
	 * Stop the monitoring of this probe.
	 */
	public void stopProbe() {
	    debugln("stopProbe(): start");

	    active = false;

	    debugln("stopProbe(): end");
	}

	/** 
	 * Set the probing parameters of the probe and make it start working
	 * @param srcIP the source node of the network link to be monitored
	 * @param dstIP the destination node of the network link to be monitored
	 * @param frequence probing frequency
	 * @return true/false, i.e., whether the setting up is successful
	 * @exception RemosProbe.ProbeException
	 */
        public boolean activate(InetAddress srcIP, InetAddress dstIP,
			     double frequence) throws ProbeException {
	    debugln("activate(): "
		+srcIP.toString() + dstIP.toString() + frequence);

	    try {
               srcId = new NodeId(srcIP.getHostName());
               dstId = new NodeId(dstIP.getHostName());
	    } catch (RemosException ex) {
		System.err.println("activate failed " + ex);
	    }

	    this.frequence = frequence;
	    this.interval = (long)(1000/frequence);

	    active = true;
	    debugln("Probe started");
	    return true;
        };

	/** 
	 * Change the probing parameters of the probe and make it start working
	 * @param srcIP the source node of the network link to be monitored
	 * @param dstIP the destination node of the network link to be monitored
	 * @param frequence probing frequency
	 * @return true/false, i.e., whether the status changing is successful
	 * @exception RemosProbe.ProbeException
	 */
        public boolean focus(InetAddress srcIP, InetAddress dstIP,
			     double frequence) throws ProbeException {
	    /* TODO:
	     * This implementation actually has no difference with activate()
	     */
	    debugln("focus(): "
		+srcIP.toString() + dstIP.toString() + frequence);

	    try {
               srcId = new NodeId(srcIP.getHostName());
              dstId = new NodeId(dstIP.getHostName());
	    } catch (RemosException ex) {
		System.err.println(ex);
	    }

	    this.frequence = frequence;
	    this.interval = (long)(1000/frequence);

	    active = true;
	    return true;
        }

	/**
	 * Not implemented.
	 */
	public void query() {
	};

	/**
	 * This is where the probing is carried out.
	 */
	public void run() {
	   debugln("run(): started");

	   try {
           	double tf = 5 ;
	   	while (true) {
		    if (active) {
			/* probe the flow performance between srcId and dstId */
           	    	Flow vFlow = myModeler.getFlow(srcId, dstId, tf);
		        debugln("get bw: "+vFlow.getBw());
			/* report the probed value, which eventually get to
			 * user application 
			 */
		        pm.report(id, vFlow.getBw());
		    }
		    sleep(interval);
	     	}
	   } catch (RemosException e) {
		System.out.println("getFlow failed " + e);
		pm.reportError(id);
	   } catch (InterruptedException e) {
		System.out.println(e);
	   }

	   debugln("run(): done.");
	};

	/**
	 * Retrieve this probe's ID
	 * @return probe ID
	 */
	public ProbeID getID() {
	    return id;	
	}

	/**
	 * Ouput debugging message
	 * @param s debugging message
	 */
        private void debugln(String s) {
           System.out.println("RemosProbe.Probe:"+s);
        }
}
