package edu.cmu.cs.able.probes.RemosProbe;

import siena.*;
import java.io.*;
import java.net.*;
import java.util.*;

/** 
 * ProbeManagerComm is a kind of server which sits on siena bus, listening
 * the probe request message sent by ProbeManagerHandle, and behave accordingly.
 */
public class ProbeManagerComm implements Notifiable {
	/** ProbeManagerComm use siena to communication with ProbeManagerHandle */
	Siena siena;
	/** The probe manager that do the real probe management work */
	ProbeManager pm;

	/** determine whether to print out the debug messages from this class */
	boolean debugf = true;

	/**
	 * Create an instance of ProbeManagerComm and starts listening
	 * on siena about the events that it is interested in.
	 */
	public static void main(String args[]) {
            HierarchicalDispatcher siena;
	    try {
		/* Initialize using siena, please refer to siena README */
                siena = new HierarchicalDispatcher();
            	siena.setMaster(args[0]);

	        ProbeManagerComm pmc = new ProbeManagerComm(siena);

		/* We only interested in those siena events with 
		 * "ProbeType = RemosProbe" and "Direction = Request".
		 */
	    	Filter f = new Filter();
	    	f.addConstraint("ProbeType", "RemosProbe");
	    	f.addConstraint("Direction", "Request");

		try {
		    /* ProbeManagerComm starts listens on siena bus */
	    	    siena.subscribe(f, pmc);
		    try {
		       System.out.println("waiting for request");

		       /* The most ugly part:
			* Sleeps for five minutes, this period of time is 
			* actually the time that the ProbeManagerComm works,
			* after that, this server will exist, and no probe
			* work will be done.
		        */
		       /* TODO:
			* make it a server, instead of sleeping
			*/
                       Thread.sleep(7200000);       
		       System.out.println("stop");
                    } catch (java.lang.InterruptedException ex) {
                       System.out.println("interrupted");
                    }
	    	} catch (SienaException ex) {
		    System.out.println("error:" + ex);
	    	}

		/* Exist point. */
		siena.unsubscribe(f, pmc);
		siena.shutdown();

	    } catch (InvalidSenderException ex) {
		System.out.println("error:" + ex);
	    } catch (IOException ex) {
		System.out.println("error:" + ex);
	    } catch (SienaException ex) {
		System.out.println("error:" + ex);
	    }

	    System.out.println("exit");
            System.exit(0);
	}

	/**
	 * Used by the main() method of this class to create a 
	 * ProbeManagerComm instance.
	 */
	ProbeManagerComm(Siena s) {
	   debugln("Constructor: start");

	   siena = s;
	   pm = new ProbeManager(this);

	   debugln("Constructor: done.");
	}
	
        /** Process the deploy request from ProbeManagerHandle, create a 
          * probe and reply with probe name
          * Message received from ProbeManageHandle should have the following 
	  * format: <p>
          *     o. "RemosProbe", "Deploy", "Request" <p>
	  * Here, "RemosProbe" is the ProbeType, all the siean message sent
	  * through siena bus should have this type, so as to differentiate
	  * with the messages sent by other types of probes; "Deploy" is the
	  * Action; and "Request" gives the message direction information, 
	  * that is, indicating that the message is sent from ProbeManageHandle
	  * to the ProbeManangeComm. <P>
	  *
          * The reply message should be one of the following two, according to
	  * whether or not the action is successful. <P>
          *     o. "RemosProbe", "Deploy", "Reply", "Succeed", "ProbeID"  <p>
          *     o. "RemosProbe", "Deploy", "Reply", "Fail" <p>
	  *
	  * @param e siena notification message
          */
	public void onDeploy(Notification e) {
	    debugln("onDeploy(): start");

	    String strTime = e.getAttribute("TimeStamp").stringValue();
	    ProbeID prbID = pm.createProbe();
	   
	    /* Construct the reply message */
            Notification reply = new Notification();
            reply.putAttribute("ProbeType", "RemosProbe");
            reply.putAttribute("Action", "Deploy");
            reply.putAttribute("Direction", "Reply");
            reply.putAttribute("TimeStamp", strTime);

	    if (prbID != null) { /* succeed */
            	reply.putAttribute("Result", "Succeed");
            	reply.putAttribute("ProbeID", prbID.toString());
		debugln("reply probe ID: "+prbID.toString());
	    }
	    else  /* fail */
            	reply.putAttribute("Result", "Fail");

            try {
                siena.publish(reply);
            } catch (Exception ex) {
                System.err.println("error:"+ex.toString());
            }

	    debugln("onDeploy(): done.");
	}

	/**
	 * Undeploy a probe
	 * @param e siean notification message
	 */
	public void onUndeploy(Notification e) {
	    debugln("onUndeploy(): start");

	    String strID = e.getAttribute("ProbeID").stringValue();
	    if (strID == null) {
		System.err.println("NULL Probe ID");
		System.exit(0);
	    }

	    ProbeID prbID = new ProbeID(strID);
	    boolean result;
     	    result = pm.destroyProbe(prbID);
 
	    Notification reply = new Notification();
            reply.putAttribute("ProbeType", "RemosProbe");
            reply.putAttribute("Action", "Undeploy");
            reply.putAttribute("Direction", "Reply");
	    if (result) 
            	reply.putAttribute("Result", "Succeed");
	    else
            	reply.putAttribute("Result", "Fail");

            try {
		debugln(reply.toString());
                siena.publish(reply);
            } catch (Exception ex) {
                System.err.println("error:"+ex.toString());
            }

	    debugln("onUndeploy(): done.");
	}

	/**
	 * Used to activate a probe.
	 * @param e siena notification message
	 */
	public void onActivate(Notification e) {
		prbConfig(e, "Activate");
	}
	
	/**
	 * Used to change the monitoring parameters of the probe
	 * @param e siena notification message
	 */
	public void onFocus(Notification e) {
		prbConfig(e, "Focus");
	}

	/**
	 * Report the probing value back to ProbeHandle
	 * @param prbID the ID of the probe that reports this value
	 * @param bw the available bandwidth from the probe
	 */
	public void sensed(ProbeID prbID, double bw) {
            Notification e = new Notification();
            e.putAttribute("ProbeType", "RemosProbe");
            e.putAttribute("ProbeID", prbID.toString());
            e.putAttribute("Action", "Sensed");
            e.putAttribute("Direction", "Reply");
            e.putAttribute("Bandwidth", Double.toString(bw));

            /* publish request */
            try {
                siena.publish(e);
                debugln("publish sensed:"+e);
            } catch (SienaException ex) {
                System.err.println("error:"+ex.toString());
	    }
	}

	/**
	 * Siena notfication method
	 * @param e siena notification message
	 */
	public void notify(Notification e) {
	     debugln(e.toString());
	     String strAction = e.getAttribute("Action").stringValue();
	     if (strAction.equals("Deploy")) {
		onDeploy(e);
	     } else if (strAction.equals("Undeploy")) {
		onUndeploy(e);
	     } else if (strAction.equals("Activate")) {
		onActivate(e);	
	     } else if (strAction.equals("Focus")) {
		onFocus(e);
	     } else {
		System.err.println("Not supported Action!");
	     }
	}

	/**
	 * Siena notfication method
	 * @param es siena notification message
	 */
	public void notify(Notification[] es) {
	}

	/**
	 * Set the working status of a probe.
	 * @param e siena notification message, including the status setting parameters
	 * @param action type of action, Activate or Focus. Currently, we do not differentiate Activate and Focus, that is, Focus has the same sementics with Activate.
	 */
	private void prbConfig(Notification e, String action) {
	    /* Abstract the information in the siena message */
	    String strProbeID = e.getAttribute("ProbeID").stringValue();
	    String strSrcIP = e.getAttribute("SourceIP").stringValue();
	    String strDstIP = e.getAttribute("DestinationIP").stringValue();
	    double frequency = e.getAttribute("Frequency").doubleValue();
	    InetAddress srcIP;
	    InetAddress dstIP;
	    boolean result;
	
	    ProbeID prbID = new ProbeID(strProbeID);
 	    try {
	    	srcIP = InetAddress.getByName(strSrcIP);
	    	dstIP = InetAddress.getByName(strDstIP);
		Probe curProbe = pm.getProbeByID(prbID);
		if (curProbe == null) {
		    System.err.println("null probe");
		    return;
		}
	    	result = curProbe.activate(srcIP, dstIP, frequency); 

		Notification reply = new Notification();
            	reply.putAttribute("ProbeType", "RemosProbe");
            	reply.putAttribute("Action", "Activate");
            	reply.putAttribute("Direction", "Reply");

	    	if (result) {
            	    reply.putAttribute("Result", "Succeed");
	    	}
	    	else 
            	    reply.putAttribute("Result", "Fail");

                siena.publish(reply);
    	    } catch (UnknownHostException ex) {
		System.err.println(ex.toString());
	    } catch (ProbeException ex) {
		System.err.println(ex.toString());
            } catch (SienaException ex) {
                System.err.println("error:"+ex.toString());
            }
	}

	/**
	 * Used to report probe execution error to application
	 * @param prbID ID of the probe that create the error
	 */
	public void reportError(ProbeID prbID) {
            Notification e = new Notification();
            e.putAttribute("ProbeType", "RemosProbe");
            e.putAttribute("ProbeID", prbID.toString());
            e.putAttribute("Action", "Fail");
            e.putAttribute("Direction", "Reply");

            /* publish request */
            try {
                siena.publish(e);
                debugln("publish failed:"+e);
            } catch (SienaException ex) {
                System.err.println("error:"+ex.toString());
	    }
	}

	/**
	 * Output debuggin message
	 * @param s debuggin message
	 */
	public void debugln(String s) {
	    System.out.println("RemosProbe.ProbeManagerComm:"+s);
	}
}

