package edu.cmu.cs.able.probes.RemosProbe;

import siena.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * ProbeManagerHandle provides the interface for application to operate on
 * the probes, such as deploy, activate, focus, undeploy, etc.
 */
public class ProbeManagerHandle implements Notifiable {
        HierarchicalDispatcher siena;
	/** pc is the representitive of application */
	BandwidthProbeConsumer pc;
	/** name of ProbeManager */
	String pmName;
	/** this list keeps all the handler created by application */
	Vector probeHandleList;

	/** this is where siena even message is stored */
	NotificationBuffer nb;

	/** controld the debugging message */
	boolean debugf = true;

	/**
	 * Initialize a new handle.
	 * @param pc the reference to the application class who calls this method
	 * @param pmName name for ProbeManager provided by applicaton
	 * @param sienaSetup the siena setup information. This actually comes from command line parameter, and is about siena server address.
	 */
	public ProbeManagerHandle(BandwidthProbeConsumer pc, String pmName,
			String sienaSetup) {
            try {
                siena = new HierarchicalDispatcher();
                siena.setMaster(sienaSetup);
            } catch (InvalidSenderException ex) {
                System.out.println("error:" + ex);
            } catch (IOException ex) {
                System.out.println("error:" + ex);
            } catch (SienaException ex) {
                System.out.println("error:" + ex);
            }

		this.pc = pc;
		this.pmName = pmName;	
		probeHandleList = new Vector();
	
		nb = new NotificationBuffer();
	}
	
	/** 
	 * Deploy a new RemosProbe, that is, create a remos probe that is
	 * ready to monitor bandwidth information for application.
	 * @return the probe handle that refer to the new probe deployed
	 */
	public ProbeHandle deploy() {
	    debugln("deploy(): start");
	    String strTime = Long.toString(System.currentTimeMillis());

	    /* Construct the request message for probe deployment */
            Notification e = new Notification();
            e.putAttribute("ProbeType", "RemosProbe");
            e.putAttribute("Action", "Deploy");
            e.putAttribute("Direction", "Request");
	    e.putAttribute("TimeStamp", strTime);
	    
	    /* Construct the filter used to get back the information used to 
	     * create probe handle 
	     */
            Filter f = new Filter();
            f.addConstraint("ProbeType", "RemosProbe");
            f.addConstraint("Action", "Deploy");
            f.addConstraint("Direction", "Reply");
            f.addConstraint("TimeStamp", strTime);

	    try {
	    	/* publish request */
            	siena.subscribe(f, nb);
		debugln("subscribe interest");
		siena.publish(e);
	        debugln("publish Deploy Request");

	        /* get reply */
		Notification reply = nb.getNotification(10000);
	    	if (reply == null) {
			debugln("getNotification return null");
			return null;
		}

       	        /* create a new probe handle */
	        ProbeHandle ph;
	    	try {
		    debugln("deploy(): get reply: "+reply);
		    ph = new ProbeHandle(pc, this, siena, reply);
	    	} catch (Exception ex) {
	   	    System.out.println("create new ProbeHandle error");
		    return null;
	        }

	        if (ph != null)  
		    probeHandleList.addElement(ph);

		/* ready for the process of error from the probe execution */
            	f.addConstraint("ProbeType", "RemosProbe");
            	f.addConstraint("Action", "Fail");
            	f.addConstraint("Direction", "Reply");
            	siena.subscribe(f, this);

	        debugln("deploy(): done.");
	        return ph;

	    } catch (SienaException ex) {
		System.err.println("error:"+ex.toString());
		return null;
    	    } catch (Exception ex) {
                System.err.println("ProbeManagerHandle get notification error");
                return null;
            }
	}

	/**
	 * Undeploy some probe. That is, stop the monitoring and release 
	 * the probe object.
	 * @param prbHdl the handle of the probe to be undeployed
	 * @return true/false to denote whether we succeed in the undeployment
	 */
	public boolean undeploy(ProbeHandle prbHdl) {
	    debugln("undeploy(): start");

	    /* Construct undeployment request message */
            Notification e = new Notification();
            e.putAttribute("ProbeType", "RemosProbe");
            e.putAttribute("Action", "Undeploy");
            e.putAttribute("Direction", "Request");
            e.putAttribute("ProbeID", prbHdl.getID());
	    
	    /* Construct the message filter used to get back the information
	     * whether or not the undeployment is successful
	     */
            Filter f = new Filter();
            f.addConstraint("ProbeType", "RemosProbe");
            f.addConstraint("Action", "Undeploy");
            f.addConstraint("Direction", "Reply");

	    /* publish request */
	    try {
            	siena.subscribe(f, nb);
		debugln("subscribe interest");
		siena.publish(e);
	        debugln("publish Deploy Request");
	    } catch (SienaException ex) {
		System.err.println("error:"+ex.toString());
	    }
		
	    /* get reply */
            try {
                Notification reply = nb.getNotification(10000);
	    	if (reply == null) {
			debugln("getNotification return null");
			return false;
		}

	        debugln("undeploy(): done.");

		/* to see whether the undeployment is good */
		if (reply.getAttribute("Result").stringValue().equals("Succeed"))
		      return true;
		else return false;

    	    } catch (Exception ex) {
                System.err.println("ProbeManagerHandle get notification error");
                return false;
            }
	}

	/**
	 * Deal with Fail message from RemosProbe, basically, what it does is
	 * to print out the message and tell user something is wrong.
	 * @param e the siena event notification about the failure message.
	 */
	public void onFail(Notification e) {
	     String strID = e.getAttribute("ProbeID").stringValue();

	     ProbeHandle prbHdl = null;
	     int i = 0;
             while (i < probeHandleList.size()) {
                if ((prbHdl = (ProbeHandle)probeHandleList.elementAt(i)) != null) {
                    if (prbHdl.getID().toString().equals(strID))
			break;
                    else i++;
                }
            }

	    if (prbHdl != null) {
		debugln("onFail: undeploy() " + prbHdl.getID().toString());
	    	undeploy(prbHdl);
	    }
	}

	/**
	 * Siena notify function.
	 * @param e refer to siena document to know the detail
	 */
        public void notify(Notification e) {
             debugln(e.toString());
             String strAction = e.getAttribute("Action").stringValue();
	     debugln("notify() "+strAction);
             if (strAction.equals("Fail")) {
                onFail(e);
             }
        }

	/**
	 * Siena notify function.
	 * @param es refer to siena document to know the detail
	 */
        public void notify(Notification[] es) {
        }

	/**
	 * Print out some debugging message for user.
	 * @param s debugging message
	 */
	public void debugln(String s) {
		System.out.println("RemosProbe.ProbeManagerHandler:"+s);
	}
}
